"use strict";

import * as Sentry from '@sentry/browser';
import * as browser from "webextension-polyfill";
import * as ewe from '@eyeo/webext-ad-filtering-solution';
import * as snippets from "@eyeo/snippets";
import { service as mlService } from "@eyeo/mlaf";
import { storageLocalGet, testStorage, isDataCorrupted } from "./storage.js";
import { detectFirstRun } from "./first-run.js";
import { Blocker, PAUSED_KEY } from "./blocker.js";
import { Telemetry } from "./telemetry.js";
import { Subscriptions } from "./subscriptions.js";
import { generateUserId } from "./utils/utils.js";

import { setValidatedMethodContext } from "../lib/validated-method.js";
import { Guardian } from "./guardian.js";
import { subscriptionsMeta } from "./subscriptions-meta.js";

const manifest = browser.runtime.getManifest();
// -----------------------------------------------------------------------
// Configuration
// -----------------------------------------------------------------------
const ADDON_NAME = "uBlock";
const ADDON_VERSION = manifest.version;

// -----------------------------------------------------------------------
// Init error reporting
// -----------------------------------------------------------------------
Sentry.init({
  dsn: import.meta.env.VITE_SENTRY_DSN,
  release: ADDON_VERSION,
  initialScope: {
    tags: { manifestVersion: manifest.manifest_version }
  },
  environment: import.meta.env.VITE_NODE_ENV,
  tracesSampleRate: 0.2,
});

let sentryUserId;
browser.storage.local.get("sentryUserId").then((result) => {
  sentryUserId = result.sentryUserId;
  if (!sentryUserId) {
    sentryUserId = generateUserId();
    browser.storage.local.set({ userId: sentryUserId });
  }

  Sentry.setUser({ id: sentryUserId });
});

// -----------------------------------------------------------------------
// Pre-start
// -----------------------------------------------------------------------
const blocker = new Blocker(ADDON_NAME, ADDON_VERSION);
const telemetry = new Telemetry(ADDON_NAME, ADDON_VERSION);
const subscriptions = new Subscriptions();
const guardian = new Guardian();

// Load up context for a real extension. This is overhead so that the front end can also run without the extension
setValidatedMethodContext({ blocker, telemetry, subscriptions, guardian });

// Load message handlers
import { setValidatedMethodBrowser } from "../lib/validated-method.js";
const browserMethods = {
  sendMessage: browser.runtime.sendMessage,
  addListener: browser.runtime.onMessage.addListener,
  getURL: browser.runtime.getURL,
  browserTabs: browser.tabs,
};
setValidatedMethodBrowser(browserMethods);
import "../api/api.js";
import "../api/qa.js";

// -----------------------------------------------------------------------
// Startup
// -----------------------------------------------------------------------

// ewe.snippets.setLibrary({
//   isolatedCode: isolatedV3Code,
//   injectedCode: injectedV3Code,
// });

const startTask = ewe.start({
  name: ADDON_NAME,
  version: ADDON_VERSION,
  bundledSubscriptions: subscriptionsMeta.subscriptions,
  bundledSubscriptionsPath: "rules",
});

const tasks = [startTask];

tasks.push(
  testStorage().catch(() => {
    isDataCorrupted(true);
  })
);

function loadSnippets() {
  ewe.snippets.setLibrary({
    injectedCode: snippets.injected,
    isolatedCode: snippets.isolated
  });
}

Promise.all(tasks).then(async (results) => {
  const eweFirstRun = results[0];
  loadSnippets();
  browser.runtime.onMessage.addListener(mlService.messageListener);
  (await ewe.filters.getMigrationErrors()).forEach((element) => {
    const { error, filter } = element;
    console.warn(`Error Message: ${error}`);
    console.warn(`Filter text: ${filter.text}`);
  });
  (await ewe.subscriptions.getMigrationErrors()).forEach((element) => {
    const { error, subscription } = element;
    console.warn(`Error Message: ${error}`);
    console.warn(`Title: ${subscription.title}, URL: ${subscription.url}`);
    console.warn();
  });
  finalize(eweFirstRun).then(() => {
    guardian.setStatus("initialized", true);
  });
});

function installListener(details) {
  switch (details.reason) {
    case 'install':
      break;
    case 'update':
      //details.previousVersion
      // remove alarms in case they were updated with this version
      browser.alarms.clearAll().then(() => {
        Promise.all([
          telemetry.scheduleStatsEvent(),
          subscriptions.scheduleSyncEvent(),
        ]).then()
      })
      break;
    case 'chrome_update':
    // fallthrough
    case 'browser_update':
      break;
    case 'shared_module_update':
      break;
  }
}

if (!browser.runtime.onInstalled.hasListener(installListener)) {
  browser.runtime.onInstalled.addListener(installListener)
}

async function finalize(eweFirstRun) {
  if (eweFirstRun) {
    await detectFirstRun(
      eweFirstRun.foundSubscriptions,
      eweFirstRun.foundStorage
    );
  }

  // Add default filter lists
  await blocker.addSubscriptions();
  // Clean up any "extra" filter lists
  await blocker.removeSubscriptions();
  // Start telemetry
  await telemetry.scheduleStatsEvent();
  // Start subscription sync
  await subscriptions.scheduleSyncEvent();
  // Remove AA
  // await ewe.filters.disable([ewe.filters.ACCEPTABLE_ADS_URL]);
  // await ewe.filters.disable([ewe.filters.ACCEPTABLE_ADS_PRIVACY_URL]);

  // If uBlock was paused on shutdown (ublock_is_paused is true), then
  // unpause / remove the allow-list all entry at startup.
  const response = await storageLocalGet(PAUSED_KEY);
  if (response[PAUSED_KEY]) {
    // Unpause after a brief delay to let things settle
    setTimeout(() => blocker.isPaused(false), 200);
  }
}

Object.assign(self, { ewe });
