// Prepares Manifest v3 subscriptions
/* eslint-env node */

import { execSync } from "child_process";
import { join } from "path";
import { readFileSync, writeFileSync } from "fs";
import { copyFile, mkdir, readdir, unlink, writeFile } from "fs/promises";

// List of v3 manifest files to patch with new ruleset data
const manifests = ["edge", "chrome"];

const cwd = join(process.cwd(), "node_modules", "@eyeo", "webext-ad-filtering-solution");
const out = join(cwd, "scriptsOutput");
const publicDir = join(".", "public");
const rulesetsDir = join(publicDir, "rulesets");

// -----------------------------------------------------
// Generate filter list rule sets
// -----------------------------------------------------
console.log("init");
console.log(execSync("npm exec subs-init", { cwd }).toString());
console.log("merge");
console.log(execSync("npm exec subs-merge", { cwd }).toString());
console.log("fetch");
console.log(execSync("npm exec subs-fetch", { cwd }).toString());
console.log("convert");
console.log(execSync("npm exec subs-convert", { cwd }).toString());
console.log("generate");
console.log(execSync("npm exec subs-generate", { cwd }).toString());
console.log("loading");

// -----------------------------------------------------
// Copy the subscriptions into the current src build
// -----------------------------------------------------

// Load `custom-subscriptions.json`, filter, and generate `public/subscriptions.json` in the extension
// Load `rulesets/rulesets.json`, filter, and add it to the mv3 manifest files
// Copy `rulesets` rulesets to the extension

// includeSubscriptions lists all the filter lists that we are bundling with
// the extension. These include:
//
// - EasyList - our primary ad blocking list
// - ABP filters - anti-circumvention filter list
// - Allow nonintrusive advertising - AA - acceptable ads
const includeSubscriptions = ["EasyList", "ABP filters", "Allow nonintrusive advertising"];
const includeIDs = [];

// disableSubscriptions are filter lists that we are bundling with the
// extension but are disabled by default. These include:
//
const disableSubscriptions = [];

// donotIncludeSubscriptions are filter lists that we are not bundling with the
// extension do to errors in the processing
//
// - Dandelion Sprout's Nordic Filters (invalid regex rule)
// - Dandelion Sprout's Nordic Filters+EasyList
const donotIncludeSubscriptions = [
  "Dandelion Sprout's Nordic Filters",
  "Dandelion Sprout's Nordic Filters+EasyList"
];
const disableIDs = [];

const subs = {
  created: new Date().toISOString(),
  subscriptions: JSON.parse(
    readFileSync(join(out, "custom-subscriptions.json")).toString()
  ),
};
console.log(`loaded ${subs.subscriptions.length} subscriptions`);

subs.subscriptions = subs.subscriptions.filter((sub) => {
  if (donotIncludeSubscriptions.indexOf(sub.title) > -1) {
    console.log(`removing subscription ${sub.title} `)
    return false;
  }
  if (includeSubscriptions.indexOf(sub.title) > -1) {
    includeIDs.push(sub.id);
    return true;
  }
  if (disableSubscriptions.indexOf(sub.title) > -1) {
    disableIDs.push(sub.id);
    return true;
  }
  if (sub.languages && sub.languages.length) {
    includeIDs.push(sub.id);
    return true;
  }
});
if (subs.subscriptions.length < 1) {
  throw "Fail: could not find subs";
}

const rulesets = JSON.parse(
  readFileSync(join(out, "rulesets", "rulesets.json")).toString()
);
console.log(`loaded ${rulesets.rule_resources.length} rulesets`);
rulesets.rule_resources = rulesets.rule_resources.filter(
  (rule) => !!subs.subscriptions.find((sub) => sub.id === rule.id)
);
if (rulesets.rule_resources.length < 1) {
  throw "Fail: could not find EasyList ruleset";
}
// We want to disable the right rulesets, the extension will enable the rulesets when installed
rulesets.rule_resources.forEach((rule) => {
  rule.enabled = includeSubscriptions.indexOf(rule.title) > -1;
  rule.path = "rulesets/" + rule.path;
});
console.log(`got filtered rulesets ${JSON.stringify(rulesets, null, "   ")}`);

writeFile(
  join(".", "public", "subscriptions.json"),
  JSON.stringify(subs, null, 2)
).then(() => {
  console.log("generated subscriptions.json");
});

writeFile(
  join(".", "src", "background", "subscriptions-meta.js"),
  "export const subscriptionsMeta = " + JSON.stringify(subs, null, 2)
).then(() => {
  console.log("generated subscriptions.json");
});

readdir(rulesetsDir)
  .then((files) => {
    const tasks = [];
    files.forEach((file) => {
      tasks.push(unlink(join(rulesetsDir, file)));
    });
    Promise.all(tasks).then(copyRules);
  })
  .catch(() => {
    mkdir(rulesetsDir).then(copyRules);
  });

manifests.forEach((manifest) => updateManifest(manifest));

function copyRules() {
  const tasks = [];
  rulesets.rule_resources.forEach((rule) => {
    tasks.push(copyFile(join(out, rule.path), join(publicDir, rule.path)));
    tasks.push(copyFile(join(out, rule.path) + ".map", join(publicDir, rule.path) + ".map"));
  });
  Promise.all(tasks).then(() => {
    console.log(`copied ${tasks.length} rulesets`);
  });
}

function updateManifest(manifest) {
  const manifestPath = join("manifest", `${manifest}.json`);
  const src = JSON.parse(readFileSync(manifestPath).toString());
  src["declarative_net_request"] = rulesets;
  writeFileSync(manifestPath, JSON.stringify(src, null, 2));
}
